"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useObservable = void 0;
var React = require("react");
var isFunction_1 = require("../utils/isFunction");
var NOOP = function () { return undefined; };
function useCollection(initial) {
    if (initial === void 0) { initial = new Map(); }
    var _a = __read(React.useState(initial), 2), value = _a[0], setValue = _a[1];
    var add = React.useCallback(function add() {
        var items = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            items[_i] = arguments[_i];
        }
        setValue(function (m) { return items.reduce(function (c, _a) {
            var _b = __read(_a, 2), k = _b[0], v = _b[1];
            return m.set(k, v);
        }, m); });
    }, []);
    var remove = React.useCallback(function remove() {
        var keys = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            keys[_i] = arguments[_i];
        }
        setValue(function (m) {
            keys.forEach(function (key) { return m.delete(key); });
            return m;
        });
    }, []);
    return { add: add, remove: remove, value: value };
}
var asObserver = function (next, error, complete) {
    if (error === void 0) { error = NOOP; }
    if (complete === void 0) { complete = NOOP; }
    return ({
        complete: complete,
        error: error,
        next: next,
        start: NOOP,
    });
};
function useObservable(value) {
    var _a = useCollection(), observers = _a.value, add = _a.add, remove = _a.remove;
    var subscribe = React.useCallback(function subscribe(listenerOrObserver) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var observer = isFunction_1.isFunction(listenerOrObserver)
            ? asObserver.apply(void 0, __spread([listenerOrObserver], args)) : listenerOrObserver;
        var closed = false;
        var subscription = {
            get closed() {
                return closed;
            },
            unsubscribe: function () {
                if (closed)
                    return;
                remove(listenerOrObserver);
                observer.complete();
                closed = true;
            },
        };
        try {
            add([listenerOrObserver, observer]);
            observer.start(subscription);
        }
        catch (e) {
            observer.error(e);
            subscription.unsubscribe();
        }
        return subscription;
    }, [add, remove]);
    var observable = React.useMemo(function getObservable() {
        var _a;
        return _a = {
                subscribe: subscribe
            },
            _a[Symbol.observable] = function () {
                return this;
            },
            _a;
    }, [subscribe]);
    var nextValue = React.useMemo(function getNextValue() {
        return __assign(__assign({}, value), observable);
    }, [observable, value]);
    var observeNext = React.useCallback(function observeNext() {
        observers.forEach(function (observer) { return observer.next(nextValue); });
    }, [observers, nextValue]);
    return [nextValue, observeNext];
}
exports.useObservable = useObservable;
